@@ -186,7 +186,7 @@ void CcdInterface::setDefaultArguments(void) {
 }
 
 double CcdInterface::initializeModel(
-		ModelData** modelData,
+		AbstractModelData** modelData,
 		CyclicCoordinateDescent** ccd,
 		AbstractModelSpecifics** model) {
 
@@ -209,7 +209,7 @@ std::string CcdInterface::getPathAndFileName(const CCDArguments& arguments, std:
 	return fileName;
 }
 
-double CcdInterface::predictModel(CyclicCoordinateDescent *ccd, ModelData *modelData) {
+double CcdInterface::predictModel(CyclicCoordinateDescent *ccd, AbstractModelData *modelData) {
 
 	struct timeval time1, time2;
 	gettimeofday(&time1, NULL);
@@ -259,7 +259,7 @@ struct OptimizationProfile {
 	bool includePenalty;
 };
 
-double CcdInterface::profileModel(CyclicCoordinateDescent *ccd, ModelData *modelData,
+double CcdInterface::profileModel(CyclicCoordinateDescent *ccd, AbstractModelData *modelData,
 		const ProfileVector& profileCI, ProfileInformationMap& profileMap,
 		int inThreads, double threshold,
 		bool overrideNoRegularization, bool includePenalty) {
@@ -362,12 +362,6 @@ double CcdInterface::profileModel(CyclicCoordinateDescent *ccd, ModelData *model
 	    }
 	};
 
-
-//         for (auto it = bounds.begin(); it != bounds.end(); ++it) {
-//
-//
-//         }
-
     if (nThreads == 1) {
         std::for_each(std::begin(bounds), std::end(bounds),
                       [&getBound, ccd](const BoundType bound) {
@@ -414,13 +408,13 @@ double CcdInterface::profileModel(CyclicCoordinateDescent *ccd, ModelData *model
 
 			if (arguments.noiseLevel >= NOISY) {
 	            std::ostringstream stream;
-				stream << "Profile: " << modelData->getColumn(index).getLabel() << " (" << lowerPt << ", "
+				stream << "Profile: " << modelData->getColumnLabel(index) << " (" << lowerPt << ", "
 						<< upperPt << ")  in " << evals;
 				logger->writeLine(stream);
 			}
 
 			ProfileInformation profile(lowerPt, upperPt, evals);
-			profileMap.insert(std::pair<IdType, ProfileInformation>(modelData->getColumn(index).getNumericalLabel(), profile));
+			profileMap.insert(std::pair<IdType, ProfileInformation>(modelData->getColumnNumericalLabel(index), profile));
 
 			// std::cerr << "Placing " << profileCI[index] << " with " << profile.lower95Bound << std::endl;
 
@@ -530,7 +524,7 @@ double CcdInterface::profileModel(CyclicCoordinateDescent *ccd, ModelData *model
 	return calculateSeconds(time1, time2);
 }
 
-double CcdInterface::diagnoseModel(CyclicCoordinateDescent *ccd, ModelData *modelData,
+double CcdInterface::diagnoseModel(CyclicCoordinateDescent *ccd, AbstractModelData *modelData,
 		double loadTime,
 		double updateTime) {
 
@@ -543,7 +537,7 @@ double CcdInterface::diagnoseModel(CyclicCoordinateDescent *ccd, ModelData *mode
 	return calculateSeconds(time1, time2);
 }
 
-double CcdInterface::logModel(CyclicCoordinateDescent *ccd, ModelData *modelData,
+double CcdInterface::logModel(CyclicCoordinateDescent *ccd, AbstractModelData *modelData,
 	    ProfileInformationMap& profileMap, bool withASE) {
 
 	struct timeval time1, time2;
@@ -601,7 +595,7 @@ SelectorType CcdInterface::getDefaultSelectorTypeOrOverride(SelectorType selecto
 
 double CcdInterface::runBoostrap(
 		CyclicCoordinateDescent *ccd,
-		ModelData *modelData,
+		AbstractModelData *modelData,
 		std::vector<double>& savedBeta) {
 	struct timeval time1, time2;
 	gettimeofday(&time1, NULL);
@@ -637,15 +631,30 @@ double CcdInterface::runFitMLEAtMode(CyclicCoordinateDescent* ccd) {
 	return calculateSeconds(time1, time2);
 }
 
-double CcdInterface::runCrossValidation(CyclicCoordinateDescent *ccd, ModelData *modelData) {
+double CcdInterface::runCrossValidation(CyclicCoordinateDescent *ccd, AbstractModelData *modelData) {
 	struct timeval time1, time2;
 	gettimeofday(&time1, NULL);
 
 	auto selectorType = getDefaultSelectorTypeOrOverride(
 		arguments.crossValidation.selectorType, modelData->getModelType());
 
-	CrossValidationSelector selector(arguments.crossValidation.fold, modelData->getPidVectorSTL(),
-			selectorType, arguments.seed, logger, error); // TODO ERROR HERE!  NOT ALL MODELS ARE SUBJECT
+	// Get possible weights
+	std::vector<double> weights = ccd->getWeights();
+
+	bool useWeights = false;
+	for (auto& w : weights) {
+		if (w != 1.0) {
+			useWeights = true;
+			break;
+		}
+	}
+
+	CrossValidationSelector selector(arguments.crossValidation.fold,
+	 		modelData->getPidVectorSTL(),
+			selectorType, arguments.seed, logger, error,
+			nullptr,
+			(useWeights ? &weights : nullptr)
+			); // TODO ERROR HERE!  NOT ALL MODELS ARE SUBJECT
 
 	AbstractCrossValidationDriver* driver;
 	if (arguments.crossValidation.useAutoSearchCV) {
